# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .i_automated import IAutomated


class IAutomatedListEditor(IAutomated):
    """ The IAutomatedListEditor interface defines the API of automated editors
    that handle lists.
    """

    def simulate_append(self, id, delay, value):
        """ Simulate the user appending a value to the list.

        :param id:
            is the (possibly scoped) identifier of the widget.  This is
            normally used in exceptions.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to append.
        """

    def simulate_move(self, id, delay, from_index, to_index):
        """ Simulate the user moving a value within the list.

        :param id:
            is the (possibly scoped) identifier of the widget.  This is
            normally used in exceptions.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param from_index:
            is the initial index of the value being moved.
        :param to_index:
            is the final index of the value being moved.
        """

    def simulate_remove(self, id, delay, index):
        """ Simulate the user removing a value from the list.

        :param id:
            is the (possibly scoped) identifier of the widget.  This is
            normally used in exceptions.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param index:
            is the index of the value to remove.
        """

    def simulate_update(self, id, delay, value, index, column=None):
        """ Simulate the user updating a value in the list.

        :param id:
            is the (possibly scoped) identifier of the widget.  This is
            normally used in exceptions.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the new value.
        :param index:
            is the index of the value to update.
        :param column:
            is the name of the attribute within the model to update.  It should
            not be specified if the list's elements are a simple type.
        """
