# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..toolkits import Toolkit


def Action(parent=None, **args):
    """ Create a toolkit specific action that is adapted to the
    :class:`~dip.ui.IAction` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the action.
    """

    from ..ui import IAction

    return _factory('action', IAction, args, parent)


def CheckBox(parent=None, **args):
    """ Create a toolkit specific check box that is adapted to the
    :class:`~dip.ui.ICheckBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the check box.
    """

    from ..ui import ICheckBox

    return _factory('check_box', ICheckBox, args, parent)


def ComboBox(parent=None, **args):
    """ Create a toolkit specific combo box that is adapted to the
    :class:`~dip.ui.IComboBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the combo box.
    """

    from ..ui import IComboBox

    return _factory('combo_box', IComboBox, args, parent)


def Dialog(parent=None, **args):
    """ Create a toolkit specific dialog that is adapted to the
    :class:`~dip.ui.IDialog` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the dialog.
    """

    from ..ui import IDialog

    return _factory('dialog', IDialog, args, parent)


def Dock(parent=None, **args):
    """ Create a toolkit specific dock that is adapted to the
    :class:`~dip.ui.IDock` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the dock.
    """

    from ..ui import IDock

    return _factory('dock', IDock, args, parent)


def FilesystemLocationEditor(parent=None, **args):
    """ Create a toolkit specific file selector that is adapted to the
    :class:`~dip.ui.IFilesystemLocationEditor` interface.  Note that
    :class:`~dip.pui.StorageLocationEditor` should normally be used instead.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the filesystem location editor.
    """

    from ..ui import IFilesystemLocationEditor

    return _factory('filesystem_location_editor', IFilesystemLocationEditor,
            args, parent)


def FloatSpinBox(parent=None, **args):
    """ Create a toolkit specific float spin box that is adapted to the
    :class:`~dip.ui.IFloatSpinBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the float spin box.
    """

    from ..ui import IFloatSpinBox

    return _factory('float_spin_box', IFloatSpinBox, args, parent)


def Form(parent=None, **args):
    """ Create a toolkit specific form that is adapted to the
    :class:`~dip.ui.IForm` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the form.
    """

    from ..ui import IForm

    return _factory('form', IForm, args, parent)


def GroupBox(parent=None, **args):
    """ Create a toolkit specific group box that is adapted to the
    :class:`~dip.ui.IGroupBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the group box.
    """

    from ..ui import IGroupBox

    return _factory('group_box', IGroupBox, args, parent)


def HBox(parent=None, **args):
    """ Create a toolkit specific horizontal box container that is adapted to
    the :class:`~dip.ui.IHBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the horizontal box container.
    """

    from ..ui import IHBox

    return _factory('h_box', IHBox, args, parent)


def Label(parent=None, **args):
    """ Create a toolkit specific label that is adapted to the
    :class:`~dip.ui.ILabel` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the label.
    """

    from ..ui import ILabel

    return _factory('label', ILabel, args, parent)


def LineEditor(parent=None, **args):
    """ Create a toolkit specific line editor that is adapted to the
    :class:`~dip.ui.ILineEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the line editor.
    """

    from ..ui import ILineEditor

    return _factory('line_editor', ILineEditor, args, parent)


def ListEditor(parent=None, **args):
    """ Create a toolkit specific list editor that is adapted to the
    :class:`~dip.ui.IListEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the list editor.
    """

    from ..ui import IListEditor

    return _factory('list_editor', IListEditor, args, parent)


def MainWindow(parent=None, **args):
    """ Create a toolkit specific main window that is adapted to the
    :class:`~dip.ui.IMainWindow` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the main window.
    """

    from ..ui import IMainWindow

    return _factory('main_window', IMainWindow, args, parent)


def Menu(parent=None, **args):
    """ Create a toolkit specific menu that is adapted to the
    :class:`~dip.ui.IMenu` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the menu.
    """

    from ..ui import IMenu

    return _factory('menu', IMenu, args, parent)


def MenuBar(parent=None, **args):
    """ Create a toolkit specific menu bar that is adapted to the
    :class:`~dip.ui.IMenuBar` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the menu bar.
    """

    from ..ui import IMenuBar

    return _factory('menu_bar', IMenuBar, args, parent)


def MessageArea(parent=None, **args):
    """ Create a toolkit specific message area that is adapted to the
    :class:`~dip.ui.IMessageArea` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the message area.
    """

    from ..ui import IMessageArea

    return _factory('message_area', IMessageArea, args, parent)


def OptionList(parent=None, **args):
    """ Create a toolkit specific option list that is adapted to the
    :class:`~dip.ui.IOptionList` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the option list.
    """

    from ..ui import IOptionList

    return _factory('option_list', IOptionList, args, parent)


def PushButton(parent=None, **args):
    """ Create a toolkit specific push button that is adapted to the
    :class:`~dip.ui.IPushButton` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the push button.
    """

    from ..ui import IPushButton

    return _factory('push_button', IPushButton, args, parent)


def RadioButtons(parent=None, **args):
    """ Create a toolkit specific set of radio buttons that is adapted to the
    :class:`~dip.ui.IRadioButtons` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the radio buttons.
    """

    from ..ui import IRadioButtons

    return _factory('radio_buttons', IRadioButtons, args, parent)


def SpinBox(parent=None, **args):
    """ Create a toolkit specific int spin box that is adapted to the
    :class:`~dip.ui.ISpinBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the int spin box.
    """

    from ..ui import ISpinBox

    return _factory('spin_box', ISpinBox, args, parent)


def Splitter(parent=None, **args):
    """ Create a toolkit specific splitter that is adapted to the
    :class:`~dip.ui.ISplitter` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the splitter.
    """

    from ..ui import ISplitter

    return _factory('splitter', ISplitter, args, parent)


def StorageLocationEditor(parent=None, **args):
    """ Create a toolkit specific file selector that is adapted to the
    :class:`~dip.ui.IStorageLocationEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the storage location editor.
    """

    from ..ui import IStorageLocationEditor

    return _factory('storage_location_editor', IStorageLocationEditor, args,
            parent)


def TabBar(parent=None, **args):
    """ Create a toolkit specific tab bar that is adapted to the
    :class:`~dip.ui.ITabBar` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the tab bar.
    """

    from ..ui import ITabBar

    return _factory('tab_bar', ITabBar, args, parent)


def TableEditor(parent=None, **args):
    """ Create a toolkit specific table editor that is adapted to the
    :class:`~dip.ui.ITableEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the table editor.
    """

    from ..ui import ITableEditor

    return _factory('table_editor', ITableEditor, args, parent)


def TextEditor(parent=None, **args):
    """ Create a toolkit specific text editor that is adapted to the
    :class:`~dip.ui.ITextEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the text editor.
    """

    from ..ui import ITextEditor

    return _factory('text_editor', ITextEditor, args, parent)


def ToolButton(parent=None, **args):
    """ Create a toolkit specific tool button that is adapted to the
    :class:`~dip.ui.IToolButton` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the tool button.
    """

    from ..ui import IToolButton

    return _factory('tool_button', IToolButton, args, parent)


def VBox(parent=None, **args):
    """ Create a toolkit specific vertical box container that is adapted to the
    :class:`~dip.ui.IVBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the vertical box container.
    """

    from ..ui import IVBox

    return _factory('v_box', IVBox, args, parent)


def ViewStack(parent=None, **args):
    """ Create a toolkit specific view stack that is adapted to the
    :class:`~dip.ui.IViewStack` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the view stack.
    """

    from ..ui import IViewStack

    return _factory('view_stack', IViewStack, args, parent)


def Wizard(parent=None, **args):
    """ Create a toolkit specific wizard that is adapted to the
    :class:`~dip.ui.IWizard` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the wizard.
    """

    from ..ui import IWizard

    return _factory('wizard', IWizard, args, parent)


def WizardPage(parent=None, **args):
    """ Create a toolkit specific wizard page that is adapted to the
    :class:`~dip.ui.IWizardPage` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the wizard page.
    """

    from ..ui import IWizardPage

    return _factory('wizard_page', IWizardPage, args, parent)


def _factory(name, interface, args, parent):
    """ Use the default toolkit to create a configured object adapted to a
    particular interface.
    """

    factory = getattr(Toolkit, name)
    obj = factory(parent)
    iobj = interface(obj)

    if args:
        # Go through any arguments setting those that correspond to interface
        # attributes and saving the rest which must therefore be object
        # specific properties.
        properties = {}

        for aname, avalue in args.items():
            if hasattr(iobj, aname):
                setattr(iobj, aname, avalue)
            else:
                properties[aname] = avalue

        iobj.configure(properties)

    return obj
