# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...model import implements, Model
from ...plugins import IPlugin, IPluginManager
from ...ui import IDisplay

from .. import IModelManagerTool


@implements(IPlugin, IDisplay)
class ModelManagerToolPlugin(Model):
    """ The ModelManagerToolPlugin class is the plugin definition for the
    default model manager tool.
    """

    # The identifier of the plugin.
    id = 'dip.shell.tools.model_manager'

    # The name of the plugin.
    name = "Model manager tool plugin"

    def configure(self, plugin_manager):
        """ This is called when the plugin is enabled to ask that it configures
        itself.

        :param plugin_manager:
            the plugin manager.
        """

        ipluginmanager = IPluginManager(plugin_manager)

        # Create the tool instance.
        from ..tools.model_manager import ModelManagerTool
        tool = ModelManagerTool()

        # Bind to the model_factories extension point.
        ipluginmanager.bind_extension_point('dip.shell.model_factories',
                IModelManagerTool(tool), 'model_factories')

        # Contribute the tool.
        ipluginmanager.contribute('dip.shell.tools', tool)
