# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QFormLayout, QLayout

from ....model import adapt
from ....ui import IForm

from ..layout_adapter import LayoutAdapter


@adapt(QFormLayout, to=IForm)
class QFormLayoutIFormAdapter(LayoutAdapter):
    """ An adapter to implement IForm for a QFormLayout. """

    @IForm.views.getter
    def views(self):
        """ Invoked to get the form's views. """

        layout = self.adaptee

        views = []
        for row in range(layout.rowCount()):
            itm = layout.itemAt(row, QFormLayout.FieldRole)
            if itm is None:
                itm = layout.itemAt(row, QFormLayout.SpanningRole)

            if itm is None:
                w = None
            else:
                w = itm.widget()
                if w is None:
                    w = itm.layout()

            views.append(w)

        return tuple(views)

    @views.setter
    def views(self, value):
        """ Invoked to set the form's views. """

        layout = self.adaptee

        # Clear the layout.
        while layout.count() > 0:
            layout.takeAt(0)

        # Populate the layout.
        for row, field in enumerate(value):
            try:
                label = self.labels[row]
            except IndexError:
                label = None

            if label is None:
                if isinstance(field, QLayout):
                    layout.setLayout(row, QFormLayout.SpanningRole, field)
                else:
                    layout.setWidget(row, QFormLayout.SpanningRole, field)
            else:
                layout.addRow(label, field)
