# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, List

from .container_factory import ContainerFactory
from .i_tab_bar import ITabBar
from .tab_page import TabPage


class TabBar(ContainerFactory):
    """ The TabBar class is a container for a number of
    :class:`~dip.ui.TabPage` sub-views.
    """

    # The contents of the view.
    contents = List(TabPage)

    # This is set if the tabs are visible.
    tabs_visible = Bool(None, allow_none=True)

    # The specific interface implemented by the view.
    view_interface = ITabBar

    def configure(self, view):
        """ Reimplemented to configure the ITabBar specific attributes.

        :param view:
            is the view to configure.
        """

        itabbar = ITabBar(view)

        if self.tabs_visible is not None:
            itabbar.tabs_visible = self.tabs_visible

        super().configure(view)
