#!/usr/bin/env python3
#
# Copyright (c) 2010 Riverbank Computing Limited.


import os
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QApplication
app = QApplication(sys.argv)


# Specify temporary exclusions (usually while refactoring).
EXCLUSIONS = ['test_shell.py']


class TestDirectory:
    """ The class that handles the running of all tests in a directory
    hierachy.
    """

    def __init__(self, test_directory):
        """ Initialise the object. """

        abs_test_directory = os.path.abspath(test_directory)

        sys.path.insert(1, os.path.dirname(abs_test_directory))

        self._abs_test_directory = abs_test_directory

    def run(self):
        """ Run the tests and return ``True`` if they were all successful. """

        suite = self._build_suite()
        runner = unittest.TextTestRunner(verbosity=1)
        result = runner.run(suite)

        return result.wasSuccessful()

    def _build_suite(self):
        """ Return a TestSuite instance containing all the tests to run. """

        suite = unittest.TestSuite()

        for dirpath, dirnames, filenames in os.walk(self._abs_test_directory):
            # Ignore directories that are not Python packages.
            if '__init__.py' not in filenames:
                del dirnames[:]
            else:
                for f in filenames:
                    if f != '__init__.py' and f.startswith('test_') and f.endswith('.py'):
                        if f in EXCLUSIONS:
                            continue

                        module_suite = self._build_module_suite(dirpath, f[:-3])
                        suite.addTest(module_suite)

        return suite

    def _build_module_suite(self, dirname, module):
        """ Return a TestSuite instance containing all the tests to from a
        particular module.
        """

        root = os.path.dirname(self._abs_test_directory)
        assert dirname.startswith(root)

        module_path = dirname[len(root):].split(os.path.sep)
        if module_path[0] == '':
            del module_path[0]

        module_path.append(module)
        module_path = '.'.join(module_path)

        loader = unittest.TestLoader()

        return loader.loadTestsFromName(module_path)


if __name__ == '__main__':
    import optparse

    # Parse the command line.
    parser = optparse.OptionParser(usage="%prog [options] [test_directory]")

    options, args = parser.parse_args()

    if len(args) > 1:
        parser.print_help()
        sys.exit(2)

    if len(args) == 1:
        test_directory_name = args[0]
    else:
        test_directory_name = os.path.join(os.path.dirname(__file__), 'tests')

    test_directory = TestDirectory(test_directory_name)
    if not test_directory.run():
        sys.exit(1)

    sys.exit()
